@@ -2,7 +2,7 @@
  * (c)LGPL2+
  *
  * LXQt - a lightweight, Qt based, desktop toolset
- * http://razor-qt.org
+ * https://lxqt.org
  *
  * Copyright: 2010-2011 Razor team
  * Authors:
@@ -36,35 +36,82 @@
 #include <QSharedData>
 #include <QTimerEvent>
 
-#include <XdgIcon>
 #include <XdgDirs>
+#if QT_VERSION < QT_VERSION_CHECK(5, 6, 0)
+#include <algorithm>
+#endif
 
-using namespace LxQt;
+using namespace LXQt;
 
-class LxQt::SettingsPrivate
+class LXQt::SettingsPrivate
 {
 public:
-    SettingsPrivate(Settings* parent):
-        mChangeTimer(0),
+    SettingsPrivate(Settings* parent, bool useXdgFallback):
+        mFileChangeTimer(0),
+        mAppChangeTimer(0),
+        mAddWatchTimer(0),
         mParent(parent)
     {
+        // HACK: we need to ensure that the user (~/.config/lxqt/<module>.conf)
+        //       exists to have functional mWatcher
+        if (!mParent->contains("__userfile__"))
+        {
+            mParent->setValue("__userfile__", true);
+#if defined(WITH_XDG_DIRS_FALLBACK)
+            if (useXdgFallback)
+            {
+                //Note: Qt doesn't support the xdg spec regarding the XDG_CONFIG_DIRS
+                //https://bugreports.qt.io/browse/QTBUG-34919
+                //(Partial) workaround: if the the user specific config file doesn't exist
+                //we try to find some system-wide configuration file and copy all settings into
+                //the user specific file
+                const QString org = mParent->organizationName();
+                const QString file_name = QFileInfo{mParent->fileName()}.fileName();
+                QStringList dirs = XdgDirs::configDirs();
+#if QT_VERSION < QT_VERSION_CHECK(5, 6, 0)
+                std::reverse(dirs.begin(), dirs.end());
+                for (auto dir_i = dirs.begin(), dir_e = dirs.end(); dir_i != dir_e; ++dir_i)
+#else // QT_VERSION
+                for (auto dir_i = dirs.rbegin(), dir_e = dirs.rend(); dir_i != dir_e; ++dir_i)
+#endif
+
+                {
+                    QDir dir{*dir_i};
+                    if (dir.cd(mParent->organizationName()) && dir.exists(file_name))
+                    {
+                        QSettings system_settings{dir.absoluteFilePath(file_name), QSettings::IniFormat};
+                        const QStringList keys = system_settings.allKeys();
+                        for (const QString & key : keys)
+                        {
+                            mParent->setValue(key, system_settings.value(key));
+                        }
+                    }
+                }
+            }
+#endif
+            mParent->sync();
+        }
+        mWatcher.addPath(mParent->fileName());
+        QObject::connect(&(mWatcher), &QFileSystemWatcher::fileChanged, mParent, &Settings::_fileChanged);
     }
 
     QString localizedKey(const QString& key) const;
 
     QFileSystemWatcher mWatcher;
-    int mChangeTimer;
+    int mFileChangeTimer;
+    int mAppChangeTimer;
+    int mAddWatchTimer;
 
 private:
     Settings* mParent;
 };
 
 
-LxQtTheme* LxQtTheme::mInstance = 0;
+LXQtTheme* LXQtTheme::mInstance = 0;
 
-class LxQt::LxQtThemeData: public QSharedData {
+class LXQt::LXQtThemeData: public QSharedData {
 public:
-    LxQtThemeData(): mValid(false) {}
+    LXQtThemeData(): mValid(false) {}
     QString loadQss(const QString& qssFile) const;
     QString findTheme(const QString &themeName);
 
@@ -76,7 +123,7 @@ public:
 };
 
 
-class LxQt::GlobalSettingsPrivate
+class LXQt::GlobalSettingsPrivate
 {
 public:
     GlobalSettingsPrivate(GlobalSettings *parent):
@@ -88,7 +135,7 @@ public:
 
     GlobalSettings *mParent;
     QString mIconTheme;
-    QString mLxQtTheme;
+    QString mLXQtTheme;
     qlonglong mThemeUpdated;
 
 };
@@ -99,17 +146,8 @@ public:
  ************************************************/
 Settings::Settings(const QString& module, QObject* parent) :
     QSettings("lxqt", module, parent),
-    d_ptr(new SettingsPrivate(this))
+    d_ptr(new SettingsPrivate(this, true))
 {
-    // HACK: we need to ensure that the user (~/.config/lxqt/<module>.conf)
-    //       exists to have functional mWatcher
-    if (!contains("__userfile__"))
-    {
-        setValue("__userfile__", true);
-        sync();
-    }
-    d_ptr->mWatcher.addPath(this->fileName());
-    connect(&(d_ptr->mWatcher), &QFileSystemWatcher::fileChanged, this, &Settings::_fileChanged);
 }
 
 
@@ -118,17 +156,8 @@ Settings::Settings(const QString& module, QObject* parent) :
  ************************************************/
 Settings::Settings(const QString &fileName, QSettings::Format format, QObject *parent):
     QSettings(fileName, format, parent),
-    d_ptr(new SettingsPrivate(this))
+    d_ptr(new SettingsPrivate(this, false))
 {
-    // HACK: we need to ensure that the user (~/.config/lxqt/<module>.conf)
-    //       exists to have functional mWatcher
-    if (!contains("__userfile__"))
-    {
-        setValue("__userfile__", true);
-        sync();
-    }
-    d_ptr->mWatcher.addPath(this->fileName());
-    connect(&(d_ptr->mWatcher), &QFileSystemWatcher::fileChanged, this, &Settings::_fileChanged);
 }
 
 
@@ -137,7 +166,7 @@ Settings::Settings(const QString &fileName, QSettings::Format format, QObject *p
  ************************************************/
 Settings::Settings(const QSettings* parentSettings, const QString& subGroup, QObject* parent):
     QSettings(parentSettings->organizationName(), parentSettings->applicationName(), parent),
-    d_ptr(new SettingsPrivate(this))
+    d_ptr(new SettingsPrivate(this, false))
 {
     beginGroup(subGroup);
 }
@@ -148,7 +177,7 @@ Settings::Settings(const QSettings* parentSettings, const QString& subGroup, QOb
  ************************************************/
 Settings::Settings(const QSettings& parentSettings, const QString& subGroup, QObject* parent):
     QSettings(parentSettings.organizationName(), parentSettings.applicationName(), parent),
-    d_ptr(new SettingsPrivate(this))
+    d_ptr(new SettingsPrivate(this, false))
 {
     beginGroup(subGroup);
 }
@@ -171,15 +200,32 @@ bool Settings::event(QEvent *event)
 {
     if (event->type() == QEvent::UpdateRequest)
     {
-        emit settingsChanged();
+        // delay the settingsChanged* signal emitting for:
+        //  - checking in _fileChanged
+        //  - merging emitting the signals
+        if(d_ptr->mAppChangeTimer)
+            killTimer(d_ptr->mAppChangeTimer);
+        d_ptr->mAppChangeTimer = startTimer(100);
     }
     else if (event->type() == QEvent::Timer)
     {
-        if(static_cast<QTimerEvent*>(event)->timerId() == d_ptr->mChangeTimer)
+        const int timer = static_cast<QTimerEvent*>(event)->timerId();
+        killTimer(timer);
+        if (timer == d_ptr->mFileChangeTimer)
         {
+            d_ptr->mFileChangeTimer = 0;
             fileChanged(); // invoke the real fileChanged() handler.
-            killTimer(d_ptr->mChangeTimer);
-            d_ptr->mChangeTimer = 0;
+        } else if (timer == d_ptr->mAppChangeTimer)
+        {
+            d_ptr->mAppChangeTimer = 0;
+            // do emit the signals
+            emit settingsChangedByApp();
+            emit settingsChanged();
+        } else if (timer == d_ptr->mAddWatchTimer)
+        {
+            d_ptr->mAddWatchTimer = 0;
+            //try to re-add filename for watching
+            addWatchedFile(fileName());
         }
     }
 
@@ -189,20 +235,31 @@ bool Settings::event(QEvent *event)
 void Settings::fileChanged()
 {
     sync();
+    emit settingsChangedFromExternal();
     emit settingsChanged();
 }
 
 void Settings::_fileChanged(QString path)
 {
-    // delay the change notification for 100 ms to avoid
-    // unnecessary repeated loading of the same config file if
-    // the file is changed for several times rapidly.
-    if(d_ptr->mChangeTimer)
-        killTimer(d_ptr->mChangeTimer);
-    d_ptr->mChangeTimer = startTimer(100);
+    // check if the file isn't changed by our logic
+    // FIXME: this is poor implementation; should we rather compute some hash of values if changed by external?
+    if (0 == d_ptr->mAppChangeTimer)
+    {
+        // delay the change notification for 100 ms to avoid
+        // unnecessary repeated loading of the same config file if
+        // the file is changed for several times rapidly.
+        if(d_ptr->mFileChangeTimer)
+            killTimer(d_ptr->mFileChangeTimer);
+        d_ptr->mFileChangeTimer = startTimer(1000);
+    }
 
+    addWatchedFile(path);
+}
+
+void Settings::addWatchedFile(QString const & path)
+{
     // D*mn! yet another Qt 5.4 regression!!!
-    // See the bug report: https://github.com/lxde/lxqt/issues/441
+    // See the bug report: https://github.com/lxqt/lxqt/issues/441
     // Since Qt 5.4, QSettings uses QSaveFile to save the config files.
     // https://github.com/qtproject/qtbase/commit/8d15068911d7c0ba05732e2796aaa7a90e34a6a1#diff-e691c0405f02f3478f4f50a27bdaecde
     // QSaveFile will save the content to a new temp file, and replace the old file later.
@@ -214,7 +271,10 @@ void Settings::_fileChanged(QString path)
     // Luckily, I found a workaround: If the file path no longer exists
     // in the watcher's files(), this file is deleted.
     if(!d_ptr->mWatcher.files().contains(path))
-        d_ptr->mWatcher.addPath(path);
+        // in some situations adding fails because of non-existing file (e.g. editting file in external program)
+        if (!d_ptr->mWatcher.addPath(path) && 0 == d_ptr->mAddWatchTimer)
+            d_ptr->mAddWatchTimer = startTimer(100);
+
 }
 
 
@@ -240,12 +300,12 @@ const GlobalSettings *Settings::globalSettings()
 
 
 /************************************************
- LC_MESSAGES value	Possible keys in order of matching
- lang_COUNTRY@MODIFIER	lang_COUNTRY@MODIFIER, lang_COUNTRY, lang@MODIFIER, lang,
+ LC_MESSAGES value      Possible keys in order of matching
+ lang_COUNTRY@MODIFIER  lang_COUNTRY@MODIFIER, lang_COUNTRY, lang@MODIFIER, lang,
                         default value
- lang_COUNTRY	        lang_COUNTRY, lang, default value
- lang@MODIFIER	        lang@MODIFIER, lang, default value
- lang	                lang, default value
+ lang_COUNTRY           lang_COUNTRY, lang, default value
+ lang@MODIFIER          lang@MODIFIER, lang, default value
+ lang                   lang, default value
  ************************************************/
 QString SettingsPrivate::localizedKey(const QString& key) const
 {
@@ -337,8 +397,8 @@ void Settings::setLocalizedValue(const QString &key, const QVariant &value)
 /************************************************
 
  ************************************************/
-LxQtTheme::LxQtTheme():
-    d(new LxQtThemeData)
+LXQtTheme::LXQtTheme():
+    d(new LXQtThemeData)
 {
 }
 
@@ -346,8 +406,8 @@ LxQtTheme::LxQtTheme():
 /************************************************
 
  ************************************************/
-LxQtTheme::LxQtTheme(const QString &path):
-    d(new LxQtThemeData)
+LXQtTheme::LXQtTheme(const QString &path):
+    d(new LXQtThemeData)
 {
     if (path.isEmpty())
         return;
@@ -374,18 +434,21 @@ LxQtTheme::LxQtTheme(const QString &path):
 /************************************************
 
  ************************************************/
-QString LxQtThemeData::findTheme(const QString &themeName)
+QString LXQtThemeData::findTheme(const QString &themeName)
 {
     if (themeName.isEmpty())
-        return "";
+        return QString();
 
     QStringList paths;
+    QLatin1String fallback(LXQT_INSTALL_PREFIX);
+
     paths << XdgDirs::dataHome(false);
     paths << XdgDirs::dataDirs();
-    // TODO/FIXME: this is fallback path for standard CMAKE_INSTALL_PREFIX
-    paths << "/usr/local/share";
 
-    foreach(QString path, paths)
+    if (!paths.contains(fallback))
+        paths << fallback;
+
+    for(const QString &path : qAsConst(paths))
     {
         QDir dir(QString("%1/lxqt/themes/%2").arg(path, themeName));
         if (dir.isReadable())
@@ -399,7 +462,7 @@ QString LxQtThemeData::findTheme(const QString &themeName)
 /************************************************
 
  ************************************************/
-LxQtTheme::LxQtTheme(const LxQtTheme &other):
+LXQtTheme::LXQtTheme(const LXQtTheme &other):
     d(other.d)
 {
 }
@@ -408,7 +471,7 @@ LxQtTheme::LxQtTheme(const LxQtTheme &other):
 /************************************************
 
  ************************************************/
-LxQtTheme::~LxQtTheme()
+LXQtTheme::~LXQtTheme()
 {
 }
 
@@ -416,7 +479,7 @@ LxQtTheme::~LxQtTheme()
 /************************************************
 
  ************************************************/
-LxQtTheme& LxQtTheme::operator=(const LxQtTheme &other)
+LXQtTheme& LXQtTheme::operator=(const LXQtTheme &other)
 {
     d = other.d;
     return *this;
@@ -426,7 +489,7 @@ LxQtTheme& LxQtTheme::operator=(const LxQtTheme &other)
 /************************************************
 
  ************************************************/
-bool LxQtTheme::isValid() const
+bool LXQtTheme::isValid() const
 {
     return d->mValid;
 }
@@ -435,7 +498,7 @@ bool LxQtTheme::isValid() const
 /************************************************
 
  ************************************************/
-QString LxQtTheme::name() const
+QString LXQtTheme::name() const
 {
     return d->mName;
 }
@@ -443,7 +506,7 @@ QString LxQtTheme::name() const
 /************************************************
 
  ************************************************/
-QString LxQtTheme::path() const
+QString LXQtTheme::path() const
 {
     return d->mPath;
 }
@@ -452,7 +515,7 @@ QString LxQtTheme::path() const
 /************************************************
 
  ************************************************/
-QString LxQtTheme::previewImage() const
+QString LXQtTheme::previewImage() const
 {
     return d->mPreviewImg;
 }
@@ -461,34 +524,20 @@ QString LxQtTheme::previewImage() const
 /************************************************
 
  ************************************************/
-QString LxQtTheme::qss(const QString& module) const
+QString LXQtTheme::qss(const QString& module) const
 {
-    QString path = QString("%1/%2.qss").arg(d->mPath, module);
-
-    QString styleSheet;
-    if (!path.isEmpty())
-        styleSheet = d->loadQss(path);
-    else
-        qWarning() << QString("QSS file %1 cannot be found").arg(path);
-
-    // Single/double click ...........................
-    Settings s("desktop");
-    bool singleClick = s.value("icon-launch-mode", "singleclick").toString() == "singleclick";
-    styleSheet += QString("QAbstractItemView {activate-on-singleclick : %1; }").arg(singleClick ? 1 : 0);
-
-    return styleSheet;
+    return d->loadQss(QStringLiteral("%1/%2.qss").arg(d->mPath, module));
 }
 
 
 /************************************************
 
  ************************************************/
-QString LxQtThemeData::loadQss(const QString& qssFile) const
+QString LXQtThemeData::loadQss(const QString& qssFile) const
 {
     QFile f(qssFile);
     if (! f.open(QIODevice::ReadOnly | QIODevice::Text))
     {
-        qWarning() << "Theme: Cannot open file for reading:" << qssFile;
         return QString();
     }
 
@@ -509,7 +558,7 @@ QString LxQtThemeData::loadQss(const QString& qssFile) const
 /************************************************
 
  ************************************************/
-QString LxQtTheme::desktopBackground(int screen) const
+QString LXQtTheme::desktopBackground(int screen) const
 {
     QString wallpaperCfgFileName = QString("%1/wallpaper.cfg").arg(d->mPath);
 
@@ -537,13 +586,13 @@ QString LxQtTheme::desktopBackground(int screen) const
 /************************************************
 
  ************************************************/
-const LxQtTheme &LxQtTheme::currentTheme()
+const LXQtTheme &LXQtTheme::currentTheme()
 {
-    static LxQtTheme theme;
+    static LXQtTheme theme;
     QString name = Settings::globalSettings()->value("theme").toString();
     if (theme.name() != name)
     {
-        theme = LxQtTheme(name);
+        theme = LXQtTheme(name);
     }
     return theme;
 }
@@ -552,27 +601,27 @@ const LxQtTheme &LxQtTheme::currentTheme()
 /************************************************
 
  ************************************************/
-QList<LxQtTheme> LxQtTheme::allThemes()
+QList<LXQtTheme> LXQtTheme::allThemes()
 {
-    QList<LxQtTheme> ret;
+    QList<LXQtTheme> ret;
     QSet<QString> processed;
 
     QStringList paths;
     paths << XdgDirs::dataHome(false);
     paths << XdgDirs::dataDirs();
 
-    foreach(QString path, paths)
+    for(const QString &path : qAsConst(paths))
     {
         QDir dir(QString("%1/lxqt/themes").arg(path));
-        QFileInfoList dirs = dir.entryInfoList(QDir::AllDirs | QDir::NoDotAndDotDot);
+        const QFileInfoList dirs = dir.entryInfoList(QDir::AllDirs | QDir::NoDotAndDotDot);
 
-        foreach(QFileInfo dir, dirs)
+        for(const QFileInfo &dir : dirs)
         {
             if (!processed.contains(dir.fileName()) &&
                  QDir(dir.absoluteFilePath()).exists("lxqt-panel.qss"))
             {
                 processed << dir.fileName();
-                ret << LxQtTheme(dir.absoluteFilePath());
+                ret << LXQtTheme(dir.absoluteFilePath());
             }
 
         }
@@ -607,10 +656,12 @@ SettingsCache::SettingsCache(QSettings *settings) :
  ************************************************/
 void SettingsCache::loadFromSettings()
 {
-   foreach (QString key, mSettings.allKeys())
-   {
-       mCache.insert(key, mSettings.value(key));
-   }
+    const QStringList keys = mSettings.allKeys();
+
+    const int N = keys.size();
+    for (int i = 0; i < N; ++i) {
+        mCache.insert(keys.at(i), mSettings.value(keys.at(i)));
+    }
 }
 
 
@@ -640,24 +691,18 @@ GlobalSettings::GlobalSettings():
 {
     if (value("icon_theme").toString().isEmpty())
     {
-        QStringList failback;
-        failback << "oxygen";
-        failback << "Tango";
-        failback << "Prudence-icon";
-        failback << "Humanity";
-        failback << "elementary";
-        failback << "gnome";
+        qWarning() << QString::fromLatin1("Icon Theme not set. Fallbacking to Oxygen, if installed");
+        const QString fallback(QLatin1String("oxygen"));
 
-
-        QDir dir("/usr/share/icons/");
-        foreach (QString s, failback)
+        const QDir dir(QLatin1String(LXQT_DATA_DIR) + QLatin1String("/icons"));
+        if (dir.exists(fallback))
         {
-            if (dir.exists(s))
-            {
-                setValue("icon_theme", s);
-                sync();
-                break;
-            }
+            setValue("icon_theme", fallback);
+            sync();
+        }
+        else
+        {
+            qWarning() << QString::fromLatin1("Fallback Icon Theme (Oxygen) not found");
         }
     }
 
@@ -682,19 +727,18 @@ void GlobalSettings::fileChanged()
     QString it = value("icon_theme").toString();
     if (d->mIconTheme != it)
     {
-        d->mIconTheme = it;
-        XdgIcon::setThemeName(it);
         emit iconThemeChanged();
     }
 
     QString rt = value("theme").toString();
     qlonglong themeUpdated = value("__theme_updated__").toLongLong();
-    if ((d->mLxQtTheme != rt) || (d->mThemeUpdated != themeUpdated))
+    if ((d->mLXQtTheme != rt) || (d->mThemeUpdated != themeUpdated))
     {
-        d->mLxQtTheme = rt;
+        d->mLXQtTheme = rt;
         emit lxqtThemeChanged();
     }
 
+    emit settingsChangedFromExternal();
     emit settingsChanged();
 }
 
