@@ -2,7 +2,7 @@
  * (c)LGPL2+
  *
  * LXQt - a lightweight, Qt based, desktop toolset
- * http://razor-qt.org
+ * https://lxqt.org
  *
  * Copyright: 2013 LXQt team
  * Authors:
@@ -89,7 +89,7 @@ bool translate(const QString &name, const QString &owner)
     QTranslator *appTranslator = new QTranslator(qApp);
 
     QStringList *paths = getSearchPaths();
-    foreach(const QString &path, *paths)
+    for(const QString &path : qAsConst(*paths))
     {
         QStringList subPaths;
 
@@ -103,7 +103,7 @@ bool translate(const QString &name, const QString &owner)
             subPaths << path;
         }
 
-        foreach(const QString &p, subPaths)
+        for(const QString &p : qAsConst(subPaths))
         {
             if (appTranslator->load(name + "_" + locale, p))
             {
